@@ -3,12 +3,9 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-import base64
-import datetime
-import hashlib
-import os
-import sys
-import unittest
+import base64, datetime, hashlib, os, sys, unittest
+from warnings import warn
+
 try:
     from urllib.parse import urlparse, parse_qsl
 except ImportError:
@@ -33,6 +30,11 @@ class HOTPExampleValuesFromTheRFC(unittest.TestCase):
         self.assertEqual(hotp.at(8), '399871')
         self.assertEqual(hotp.at(9), '520489')
 
+    def test_invalid_input(self):
+        hotp = pyotp.HOTP('GEZDGNBVGY3TQOJQGEZDGNBVGY3TQOJQ')
+        with self.assertRaises(ValueError):
+            hotp.at(-1)
+
     def test_verify_otp_reuse(self):
         hotp = pyotp.HOTP('GEZDGNBVGY3TQOJQGEZDGNBVGY3TQOJQ')
         self.assertTrue(hotp.verify('520489', 9))
@@ -81,14 +83,14 @@ class HOTPExampleValuesFromTheRFC(unittest.TestCase):
 
         hotp = pyotp.HOTP(key, digits=8)
         url = urlparse(
-            hotp.provisioning_uri('baco@peperina', issuer_name='FooCorp',
+            hotp.provisioning_uri('baco@peperina', issuer_name='Foo Corp',
                                   initial_count=10))
         self.assertEqual(url.scheme, 'otpauth')
         self.assertEqual(url.netloc, 'hotp')
-        self.assertEqual(url.path, '/FooCorp:baco%40peperina')
+        self.assertEqual(url.path, '/Foo%20Corp:baco%40peperina')
         self.assertEqual(dict(parse_qsl(url.query)),
                          {'secret': 'c7uxuqhgflpw7oruedmglbrk7u6242vb',
-                          'counter': '10', 'issuer': 'FooCorp',
+                          'counter': '10', 'issuer': 'Foo Corp',
                           'digits': '8'})
 
     def test_other_secret(self):
@@ -137,9 +139,9 @@ class TOTPExampleValuesFromTheRFC(unittest.TestCase):
         for digest, secret in self.RFC_VALUES:
             totp = pyotp.TOTP(base64.b32encode(secret), 8, digest)
             for utime, code in self.RFC_VALUES[(digest, secret)]:
-                # 32-bit platforms use native functions to handle timestamps, so they fail this test
-                # (and will fail after 19 January 2038)
                 if utime > sys.maxsize:
+                    warn("32-bit platforms use native functions to handle timestamps, so they fail this test" +
+                         " (and will fail after 19 January 2038)")
                     continue
                 value = totp.at(utime)
                 msg = "%s != %s (%s, time=%d)"
@@ -165,6 +167,14 @@ class TOTPExampleValuesFromTheRFC(unittest.TestCase):
         with Timecop(1297553958 + 30):
             self.assertFalse(totp.verify('102705'))
 
+    def test_input_before_epoch(self):
+        totp = pyotp.TOTP('GEZDGNBVGY3TQOJQGEZDGNBVGY3TQOJQ')
+        # -1 and -29.5 round down to 0 (epoch)
+        self.assertEqual(totp.at(-1), '755224')
+        self.assertEqual(totp.at(-29.5), '755224')
+        with self.assertRaises(ValueError):
+            totp.at(-30)
+
     def test_validate_totp_with_digit_length(self):
         totp = pyotp.TOTP('GEZDGNBVGY3TQOJQGEZDGNBVGY3TQOJQ')
         with Timecop(1111111111):
